@@ -2,6 +2,8 @@ from .BeforeAfter import BeforeAfter
 from .Burger import Burger
 from .DeferScript import DeferScript
 from .Download import Download
+from .EventListener import EventListener
+from .EventSource import EventSource
 from .Keyboard import Keyboard
 from .Lottie import Lottie
 from .Mermaid import Mermaid
@@ -15,6 +17,8 @@ __all__ = [
     "Burger",
     "DeferScript",
     "Download",
+    "EventListener",
+    "EventSource",
     "Keyboard",
     "Lottie",
     "Mermaid",
